package dev.adriankuta.pixabay.data.di

import dagger.Binds
import dagger.Module
import dagger.Provides
import dagger.hilt.InstallIn
import dagger.hilt.components.SingletonComponent
import dev.adriankuta.pixabay.data.BuildConfig
import dev.adriankuta.pixabay.data.network.KtorPixabayService
import dev.adriankuta.pixabay.data.network.PixabayService
import io.ktor.client.HttpClient
import io.ktor.client.engine.android.Android
import io.ktor.client.plugins.contentnegotiation.ContentNegotiation
import io.ktor.client.plugins.defaultRequest
import io.ktor.client.plugins.logging.Logging
import io.ktor.client.plugins.resources.Resources
import io.ktor.http.URLProtocol
import io.ktor.serialization.kotlinx.json.json
import kotlinx.serialization.json.Json
import javax.inject.Singleton

@InstallIn(SingletonComponent::class)
@Module
internal abstract class NetworkModule {

    @Singleton
    @Binds
    abstract fun provideKtorApi(ktorPixabayService: KtorPixabayService): PixabayService

    companion object {
        @Singleton
        @Provides
        fun provideKtorClient(): HttpClient {
            val httpClient = HttpClient(Android) {
                install(ContentNegotiation) {
                    json(
                        Json {
                            ignoreUnknownKeys = true
                        }
                    )
                }
                install(Resources)
                install(Logging)
                defaultRequest {
                    url {
                        protocol = URLProtocol.HTTP
                        host = "pixabay.com"
                        parameters.append("key", BuildConfig.PIXABAY_API_KEY)
                    }
                }
                engine {
                    connectTimeout = 100_000
                    socketTimeout = 100_000
                }
            }
            return httpClient
        }

    }
}